/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007  Arto Karppinen (arto.karppinen@iki.fi),
 * Harri Vattulainen (t5vaha01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_config.h" 
#include "irreco_button.h"

/*
 * Configuration saving and reading.
 */

#define IRRECO_CONFIG_GROUP_LEN	20

gchar* irreco_config_layout_file	= "layouts.conf";
gchar* irreco_config_active_layout_file	= "active-layout.conf";
gchar* irreco_config_backend_file	= "backends.conf";
gchar* irreco_config_cmd_chain_file	= "command-chains.conf";




/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
#if 0
void irreco_config_read_style_from_keyfile(IrrecoData * irreco_data, 
					   IrrecoKeyFile *keyfile, 
					   gboolean editable);
#endif
void irreco_config_read_button_from_keyfile(IrrecoData * irreco_data, 
					    IrrecoKeyFile * keyfile);
void irreco_config_read_hardkey_from_keyfile(IrrecoData * irreco_data, 
					     IrrecoKeyFile * keyfile);
void irreco_config_read_layout_from_keyfile(IrrecoData * irreco_data, 
					    IrrecoKeyFile * keyfile);
void irreco_config_read_command_from_keyfile(IrrecoData * irreco_data, 
					     IrrecoKeyFile * keyfile);
void irreco_config_save_layout_to_keyfile(GKeyFile * keyfile, 
					  IrrecoButtonLayout * layout,
					  gint * layout_index,
					  gint * button_index,
					  gint * command_index,
					  gint * hardkey_index);
void irreco_config_save_hardkey_to_file(IrrecoHardkeyMap *hardkey_map,
					guint             keyval,
					IrrecoCmdChainId  chain_id,
					IrrecoCmdChain   *chain,
					gpointer          user_data);
void irreco_config_save_button_to_keyfile(GKeyFile * keyfile, 
					  IrrecoButton * button,
					  gint * button_index,
					  gint * command_index);
void irreco_config_save_cmd_chain_to_keyfile(GKeyFile * keyfile, 
					     IrrecoButton * button,
					     gint * command_index);
void irreco_config_read_backend_from_keyfile(
					 IrrecoBackendManager * backend_manager,
					 IrrecoKeyFile * keyfile);



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Layout reading.                                                            */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Read bstyleconf file from directory.
 *
 * One subdirectory may contain only one IrrecoButtonStyle configuration and 
 * "button-style" group.
 */
#if 0
void irreco_config_read_styles_from_dir_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoData * irreco_data = (IrrecoData*) dir_data->user_data_1;
	IrrecoKeyFile *keyfile;
	IRRECO_ENTER

	keyfile = irreco_keyfile_create(dir_data->directory, 
					 dir_data->filepath, 
					 "button-style");
	if (keyfile != NULL) {
		irreco_config_read_style_from_keyfile(irreco_data, keyfile, 
						      FALSE);
		irreco_keyfile_destroy(keyfile);
	}

	IRRECO_RETURN
}
#endif
/**
 * Reads buttons-folder inside themefolder
 *
 * This function uses irreco_dir_foreach() which calls
 * irreco_config_read_styles_from_dir_foreach()
 */
#if 0
void irreco_config_read_buttons_from_dir_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoDirForeachData button_styles;
	GString * directory = g_string_new("");
	IRRECO_ENTER
	
	g_string_printf(directory, "%s/buttons/", dir_data->filepath);
	IRRECO_DEBUG("Directory = %s\n", directory->str);
	button_styles.directory = directory->str;

	button_styles.filesuffix = "bstyleconf";
	button_styles.user_data_1 = dir_data->user_data_1;

	irreco_dir_foreach_subdirectories(&button_styles, 
			   irreco_config_read_styles_from_dir_foreach);
	
	g_string_free(directory, TRUE);
	directory = NULL;
	IRRECO_RETURN
}

#endif

/*
 * Reads configs from subdirectories ONLY
 */
/*
void irreco_config_read_styles_from_dir(IrrecoData * irreco_data, 
					const gchar *dir)
{
	IrrecoDirForeachData dir_data;
	IRRECO_ENTER
	
	dir_data.directory = dir;
	dir_data.filesuffix = ".bstyleconf";
	dir_data.user_data_1 = irreco_data;
	irreco_dir_foreach_subdirectories(&dir_data, 
			   irreco_config_read_styles_from_dir_foreach);

	IRRECO_RETURN
}
*/

/**
 * Reads themefolders from subdirectories
 *
 * This function uses irreco_dir_foreach() which calls
 * irreco_config_read_buttons_from_dir_foreach()
 */
#if 0
void irreco_config_read_themes_from_dir(IrrecoData * irreco_data, 
					const gchar *dir)
{
	IrrecoDirForeachData dir_data;
	IRRECO_ENTER
	
	dir_data.directory = dir;
	dir_data.filesuffix = "";
	dir_data.user_data_1 = irreco_data;
	irreco_dir_foreach(&dir_data, 
			   irreco_config_read_buttons_from_dir_foreach);

	IRRECO_RETURN
}
#endif
#if 0
void irreco_config_read_style_from_file(IrrecoData * irreco_data, 
					const gchar *file)
{
	gchar *dir;
	IrrecoKeyFile *keyfile;
	IRRECO_ENTER
	
	dir = g_path_get_dirname(file);
	keyfile = irreco_keyfile_create(dir, file, "button-style");
	if (keyfile != NULL) {
		irreco_config_read_style_from_keyfile(irreco_data, keyfile, 
						      FALSE);
		irreco_keyfile_destroy(keyfile);
	}
	g_free(dir);

	IRRECO_RETURN
}
#endif
gboolean irreco_config_read_layouts(IrrecoData * irreco_data)
{
	guint i;
	IrrecoKeyFile *keyfile = NULL;
	gchar *config_dir = NULL;
	gchar *config_file = NULL;
	gchar** groups = NULL;
	gsize group_count = 0;
	IRRECO_ENTER
	
	config_dir = irreco_get_config_dir("irreco");
	config_file = irreco_get_config_file("irreco", 
					     irreco_config_layout_file);
	keyfile = irreco_keyfile_create(config_dir, config_file, NULL);
	if (config_dir == NULL || config_file == NULL || keyfile == NULL) {
		g_free(config_dir);
		g_free(config_file);
		irreco_keyfile_destroy(keyfile);
		IRRECO_RETURN_BOOL(FALSE);
	}
	
	IRRECO_PRINTF("Reading layout configuration file \"%s\"\n", 
		      config_file);
	groups = g_key_file_get_groups(keyfile->keyfile, &group_count);
	
	/* Read layouts. */
	for (i = 0; i < group_count; i++) {
		gchar *group = groups[i];
		if (g_str_has_prefix(group, "layout")) {
			IRRECO_PRINTF("Reading layout group \"%s\"\n", group);
			irreco_keyfile_set_group(keyfile, group);
			irreco_config_read_layout_from_keyfile(irreco_data, 
							       keyfile);
		}
	}
	
	/* Read Buttons. */
	for (i = 0; i < group_count; i++) {
		gchar *group = groups[i];
		if (g_str_has_prefix(group, "button")) {
			IRRECO_PRINTF("Reading button group \"%s\"\n", group);
			irreco_keyfile_set_group(keyfile, group);
			irreco_config_read_button_from_keyfile(irreco_data, 
							       keyfile);
		}
	}
	
	/* Read Hardkeys. */
	for (i = 0; i < group_count; i++) {
		gchar *group = groups[i];
		if (g_str_has_prefix(group, "hardkey")) {
			IRRECO_PRINTF("Reading hardkey group \"%s\"\n", group);
			irreco_keyfile_set_group(keyfile, group);
			irreco_config_read_hardkey_from_keyfile(irreco_data, 
							        keyfile);
		}
	}
	
	/*
	 * Read LEGACY command group configuration.
	 */
	{
	gchar *chain_config = irreco_get_config_file(
		"irreco", irreco_config_cmd_chain_file);
	if (irreco_file_exists(chain_config) == FALSE) {
		for (i = 0; i < group_count; i++) {
			gchar *group = groups[i];
			if (g_str_has_prefix(group, "command")) {
				IRRECO_PRINTF("Reading command group \"%s\"\n", 
					      group);
				irreco_keyfile_set_group(keyfile, group);
				irreco_config_read_command_from_keyfile(
					irreco_data, keyfile);
			}
		}	
	}
	g_free(chain_config);
	}
	
	/* Read command chains. */
	irreco_cmd_chain_manager_from_config(irreco_data);
	
	g_strfreev(groups);
	irreco_keyfile_destroy(keyfile);
	g_free(config_dir);
	g_free(config_file);
	IRRECO_RETURN_BOOL(TRUE);
}

#if 0
void irreco_config_read_style_from_keyfile(IrrecoData * irreco_data, 
					   IrrecoKeyFile *keyfile, 
					   gboolean editable) 
{
	IrrecoButtonStyle* style;
	IRRECO_ENTER
	
	style = irreco_button_style_create(
		irreco_data, NULL, NULL, NULL, TRUE, editable);
	
	/* Required fields. */			
	if (!irreco_keyfile_get_str(keyfile, "name", &style->name) ||
	    !irreco_keyfile_get_path(keyfile, "up", &style->up)) {
		IRRECO_PRINTF("Could not read style from group \"%s\"\n", 
			   keyfile->group);
		irreco_button_style_destroy(style);
		IRRECO_RETURN
	}
	IRRECO_PRINTF("Name = \"%s\"\n", style->name);
	IRRECO_PRINTF("Up = \"%s\"\n", style->up);
	
	/* Optional fields. */
	irreco_keyfile_get_path(keyfile, "down", &style->down);
	IRRECO_PRINTF("Down = \"%s\"\n", style->down);
	irreco_keyfile_get_bool(keyfile, "allow-text", &style->allow_text);
	IRRECO_PRINTF("Allow-text = \"%i\"\n", style->allow_text);
	irreco_keyfile_get_str(keyfile, "text-format-up", &style->text_format_up);
	IRRECO_PRINTF("text-format-up = \"%s\"\n", style->text_format_up);
	irreco_keyfile_get_str(keyfile, "text-format-down", 
			     &style->text_format_down);
	IRRECO_PRINTF("text-format-down = \"%s\"\n", style->text_format_down);
	irreco_keyfile_get_int(keyfile, "text-padding", &style->text_padding);
	IRRECO_PRINTF("Text-padding = \"%i\"\n", style->text_padding);
	
	/* Attempt to add the style to the style array. */
	if (!irreco_string_table_add(irreco_data->button_style_array, style->name, 
				  style)) {
		IRRECO_ERROR("Dublicate style name \"%s\"\n", style->name);
		irreco_button_style_destroy(style);
	}

	IRRECO_RETURN
}
#endif
void irreco_config_read_layout_from_keyfile(IrrecoData * irreco_data, 
					    IrrecoKeyFile * keyfile)
{
	gchar *name = NULL;
	IrrecoButtonLayout *layout = NULL;
	GdkColor color;
	gint type;
	gchar *image = NULL;
	IRRECO_ENTER
	
	/* Required fields. */	
	if (!irreco_keyfile_get_str(keyfile, "name", &name)) IRRECO_RETURN
	layout = irreco_button_layout_create(NULL, 
					     irreco_data->cmd_chain_manager);
	irreco_button_layout_set_name(layout, name);
	g_free(name);
	name = NULL;
	
	if (irreco_keyfile_get_int(keyfile, "bg-type", &type)) {
		switch (type) {
		case IRRECO_BACKGROUND_DEFAULT:
		case IRRECO_BACKGROUND_COLOR:
		case IRRECO_BACKGROUND_IMAGE:
			irreco_button_layout_set_bg_type(layout, type);
			break;
			
		default:
			IRRECO_ERROR("Unknown background type \"%i\".\n", 
				     type);
			break;
		}
	}
	
	if (irreco_keyfile_get_str(keyfile, "bg-image", &image)) {
		irreco_button_layout_set_bg_image(layout, image);
		g_free(image);
		image = NULL;
	}
	
	color.pixel = 0;
	if (irreco_keyfile_get_uint16(keyfile, "bg-color-red", &color.red) && 
	    irreco_keyfile_get_uint16(keyfile, "bg-color-green", &color.green)&&
	    irreco_keyfile_get_uint16(keyfile, "bg-color-blue", &color.blue)) {
		irreco_button_layout_set_bg_color(layout, &color);
	}
	
	if (!irreco_string_table_add(irreco_data->irreco_layout_array, 
				    irreco_button_layout_get_name(layout), 
				    layout)) {
		IRRECO_ERROR("Layout \"%s\" already exists.", 
			     irreco_button_layout_get_name(layout));
		irreco_button_layout_destroy(layout);
	}

	IRRECO_RETURN
}

void irreco_config_read_button_from_keyfile(IrrecoData * irreco_data, 
					    IrrecoKeyFile * keyfile)
{
	gchar *layout_name = NULL;
	gchar *style_name = NULL;
	gchar *title = NULL;
	gchar *command = NULL;
	IrrecoCmdChainId chain_id;
	gint x;
	gint y;
	
	IrrecoButton *button = NULL;
	IrrecoButtonLayout *layout = NULL;
	IrrecoButtonStyle *style = NULL;
	IRRECO_ENTER
	
	/* Required fields. */	
	if (!irreco_keyfile_get_int(keyfile, "x", &x) ||
	    !irreco_keyfile_get_int(keyfile, "y", &y) ||
	    !irreco_keyfile_get_str(keyfile, "layout", &layout_name)) {
		IRRECO_RETURN
	}
	IRRECO_PRINTF("Position: x%i y%i\n", x, y);
	
	/* Find layout. */
	if (!irreco_string_table_get(irreco_data->irreco_layout_array, 
				    layout_name, (gpointer *) &layout)) {
		IRRECO_ERROR("Button uses unknown layout \"%s\"\n", layout_name);
		g_free(layout_name);
		IRRECO_RETURN
	}
	g_free(layout_name);
	IRRECO_PRINTF("Style: \"%s\"\n", 
		      irreco_button_layout_get_name(layout));
	
	/* Find style. */
	irreco_keyfile_get_str(keyfile, "style", &style_name);
	if (style_name != NULL) {
		if (irreco_theme_manager_get_button_style(
						irreco_data->theme_manager,
						style_name, &style)) {
			IRRECO_PRINTF("Style: \"%s\"\n", style->name);
		} else {
			IRRECO_ERROR("Button uses unknown style \"%s\"\n",
				     style_name);
		}
#if 0
		if (irreco_string_table_get(irreco_data->button_style_array, 
					    style_name, (gpointer *) &style)) {
			IRRECO_PRINTF("Style: \"%s\"\n", style->name);
		} else {
			IRRECO_ERROR("Button uses unknown style \"%s\"\n", 
				     style_name);
		}
#endif
		g_free(style_name);
	}

	/* Get title and command. */
	irreco_keyfile_get_str(keyfile, "title", &title);
	IRRECO_PRINTF("title: \"%s\"\n", title);
	
	button = irreco_button_create(layout, x, y, title, style, 
				      irreco_data->cmd_chain_manager);
				      
	if (irreco_keyfile_get_int(keyfile, "command-chain-id", &chain_id)) {
		irreco_button_set_cmd_chain_id(button, chain_id);
	}
	
	g_free(title);
	g_free(command);
	IRRECO_RETURN
}

void irreco_config_read_hardkey_from_keyfile(IrrecoData * irreco_data, 
					     IrrecoKeyFile * keyfile)
{
	IrrecoButtonLayout *layout = NULL;
	gchar *layout_name = NULL;
	IrrecoCmdChainId id;
	guint keyval;
	gchar *hardkey_str = NULL;
	IRRECO_ENTER
	
	/* Required fields. */	
	if (!irreco_keyfile_get_str(keyfile, "layout", &layout_name) ||
	    !irreco_keyfile_get_int(keyfile, "chain-id", &id) ||
	    !irreco_keyfile_get_uint(keyfile, "key-value", &keyval)) {
		g_free(layout_name);
		IRRECO_RETURN
	}
	
	/* Find layout. */
	if (!irreco_string_table_get(irreco_data->irreco_layout_array, 
				     layout_name, (gpointer *) &layout)) {
		IRRECO_ERROR("Hardkey uses unknown layout \"%s\"\n", 
			     layout_name);
		g_free(layout_name);
		IRRECO_RETURN
	}
	g_free(layout_name);
	
	hardkey_str = irreco_hardkey_to_str(keyval);
	IRRECO_PRINTF("Associating layout \"%s\" hardkey \"%s\" with "
		      "command chain \"%i\".\n", 
		      irreco_button_layout_get_name(layout),
		      hardkey_str, id);
	g_free(hardkey_str);
	
	irreco_hardkey_map_assosiate_chain_with_id(layout->hardkey_map, 
						   keyval, id);
	IRRECO_RETURN
}

/**
 * Support LEGACY configuration.
 *
 * This still exists in order to read layout configs from old versions of 
 * irreco. 
 *
 * TODO: Remove sometime next year.
 */
void irreco_config_read_command_from_keyfile(IrrecoData * irreco_data, 
					     IrrecoKeyFile * keyfile)
{
	gint button_index;
	IrrecoCmdType type;
	gchar *layout_name = NULL;
	gchar *type_name = NULL;
	
	IrrecoCmd *command;
	IrrecoButton *button;
	IrrecoButtonLayout *layout;
	IRRECO_ENTER
	
	if (!irreco_keyfile_get_str(keyfile, "layout", &layout_name) ||
	    !irreco_keyfile_get_str(keyfile, "type",   &type_name) ||
	    !irreco_keyfile_get_int(keyfile, "button", &button_index)) {
		g_free(layout_name);
		g_free(type_name);
		IRRECO_RETURN
	}
	
	/* Convert type name to type ID. */
	if ((type = irreco_cmd_str_to_type(type_name)) == -1) {
		IRRECO_ERROR("Unknown command type \"%s\"\n", type_name);
		g_free(layout_name);
		g_free(type_name);
		IRRECO_RETURN
	}
	g_free(type_name);
	
	/* Find layout. */
	if (!irreco_string_table_get(irreco_data->irreco_layout_array,
				     layout_name, (gpointer *) &layout)) {
		IRRECO_ERROR("Unknown layout \"%s\"\n", layout_name);
		g_free(layout_name);
		IRRECO_RETURN
	}
	g_free(layout_name);
	
	/* Find Button. */
	if (button_index >= layout->button_array->len) {
		IRRECO_ERROR("Invalid button index \"%i\"\n", button_index);
		IRRECO_RETURN
	}
	button = g_ptr_array_index(layout->button_array, button_index);
	
	/* Append command to button command chain. */
	switch (type) {
	case IRRECO_COMMAND_NONE:
	case IRRECO_COMMAND_NEXT_REMOTE:
	case IRRECO_COMMAND_PREVIOUS_REMOTE:
	case IRRECO_COMMAND_FULLSCREEN_TOGGLE:
		command = irreco_cmd_create();
		irreco_cmd_set_builtin(command, type);
		irreco_cmd_chain_append(irreco_button_get_cmd_chain(button), 
					command);
		break;
		
	case IRRECO_COMMAND_BACKEND: {
		gchar *instance_name = NULL;
		gchar *device_name = NULL;
		gchar *command_name = NULL;
		IrrecoBackendInstance * backend_instance;
	
		if (!irreco_keyfile_get_str(keyfile, "instance-name", 
					     &instance_name) ||
		    !irreco_keyfile_get_str(keyfile, "device-name", 
		    			     &device_name) ||
		    !irreco_keyfile_get_str(keyfile, "command-name",
		    			     &command_name)) {
			g_free(instance_name);
			g_free(device_name);
			g_free(command_name);
			IRRECO_RETURN
		}
		
		if (!irreco_backend_manager_find_instance(
			irreco_data->irreco_backend_manager,
			instance_name,
			&backend_instance)) {
			IRRECO_ERROR("Could not find instance with name "
				     "\"%s\"\n", instance_name);
			IRRECO_RETURN
		}
		command = irreco_cmd_create();
		irreco_cmd_set_backend(command, 
				       backend_instance,
				       device_name,
				       command_name);
		irreco_cmd_chain_append(irreco_button_get_cmd_chain(button), 
					command);
		
		g_free(instance_name);
		g_free(device_name);
		g_free(command_name);
		} break;
	
	case IRRECO_COMMAND_SHOW_LAYOUT: {
		gchar *show_layout;
		if (!irreco_keyfile_get_str(keyfile, "show-layout", 
					     &show_layout)) {
			IRRECO_RETURN
		}
		
		command = irreco_cmd_create();
		irreco_cmd_set_layout(command, show_layout);
		irreco_cmd_chain_append(irreco_button_get_cmd_chain(button),
					command);
		g_free(show_layout);
		} break;
		
	case IRRECO_COMMAND_WAIT: {
		guint delay;
		if (!irreco_keyfile_get_uint(keyfile, "delay", &delay)) {
			IRRECO_RETURN
		}
		
		command = irreco_cmd_create();
		irreco_cmd_set_wait(command, delay);
		irreco_cmd_chain_append(irreco_button_get_cmd_chain(button),
					command);
		} break;
	}

	IRRECO_RETURN
}




/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Layout saving.                                                             */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Save layout.
 *
 * Since GKeyFile does not allow two groups with the same name,
 * layout_index and button_index variables are used to make sure that each 
 * button and layout group has unique name. The index is ignored when reading 
 * the layout config.
 */
gboolean irreco_config_save_layouts(IrrecoData * irreco_data)
{
	gint layout_index = 1;
	gint button_index = 1;
	gint command_index = 1;
	gint hardkey_index = 1;
	GKeyFile *keyfile;
	IrrecoStringTable *table;
	gboolean success;
	IRRECO_ENTER
	
	irreco_cmd_chain_manager_to_config(irreco_data->cmd_chain_manager);
	
	keyfile = g_key_file_new();
	table = irreco_data->irreco_layout_array;
	IRRECO_STRING_TABLE_FOREACH(table, key, IrrecoButtonLayout *, layout)
		irreco_config_save_layout_to_keyfile(keyfile,
						     layout, 
						     &layout_index, 
						     &button_index,
						     &command_index,
						     &hardkey_index);
		layout_index++;
	IRRECO_STRING_TABLE_FOREACH_END
	
	success = irreco_gkeyfile_write_to_config_file(keyfile, "irreco", 
						     irreco_config_layout_file);
	g_key_file_free(keyfile);
	IRRECO_RETURN_BOOL(success);
}

typedef struct _IrrecoConfigSaveHardkeyData IrrecoConfigSaveHardkeyData;
struct _IrrecoConfigSaveHardkeyData {
	GKeyFile *keyfile;
	gint *hardkey_index;
	IrrecoButtonLayout *layout;
};

void irreco_config_save_layout_to_keyfile(GKeyFile * keyfile, 
					  IrrecoButtonLayout * layout,
					  gint * layout_index,
					  gint * button_index,
					  gint * command_index,
					  gint * hardkey_index)
{
	GdkColor *color;
	gchar group[IRRECO_CONFIG_GROUP_LEN];
	IrrecoConfigSaveHardkeyData hardkey_data;
	IRRECO_ENTER
	
	/* Save layout. */
	g_assert(irreco_button_layout_get_name(layout) != NULL);
	g_snprintf(group, IRRECO_CONFIG_GROUP_LEN, "layout-%i", *layout_index);
	irreco_gkeyfile_set_string(keyfile, group, "name", 
				 irreco_button_layout_get_name(layout));
	g_key_file_set_integer(keyfile, group, "bg-type", 
			       irreco_button_layout_get_bg_type(layout));		      
	irreco_gkeyfile_set_string(keyfile, group, "bg-image", 
				 irreco_button_layout_get_bg_image(layout));
	color = irreco_button_layout_get_bg_color(layout);
	g_key_file_set_integer(keyfile, group, "bg-color-red", color->red);
	g_key_file_set_integer(keyfile, group, "bg-color-green", color->green);
	g_key_file_set_integer(keyfile, group, "bg-color-blue", color->blue);
	
	/* Save buttons. */
	irreco_button_layout_reindex(layout);
	IRRECO_PTR_ARRAY_FORWARDS(layout->button_array, IrrecoButton *, button)
		irreco_config_save_button_to_keyfile(keyfile, 
						     button, 
						     button_index,
						     command_index);
		*button_index = *button_index + 1;
	IRRECO_PTR_ARRAY_FORWARDS_END

	/* Save Hardkeys. */
	hardkey_data.keyfile = keyfile;
	hardkey_data.hardkey_index = hardkey_index;
	hardkey_data.layout = layout;
	irreco_hardkey_map_chain_foreach(layout->hardkey_map, 
					 irreco_config_save_hardkey_to_file,
					 &hardkey_data);
	IRRECO_RETURN
}

void irreco_config_save_hardkey_to_file(IrrecoHardkeyMap *hardkey_map,
					guint             keyval,
					IrrecoCmdChainId  chain_id,
					IrrecoCmdChain   *chain,
					gpointer          user_data)
{
	gchar group[IRRECO_CONFIG_GROUP_LEN];
	IrrecoConfigSaveHardkeyData *hardkey_data;
	IRRECO_ENTER
	
	hardkey_data = (IrrecoConfigSaveHardkeyData*) user_data;
	
	g_snprintf(group, IRRECO_CONFIG_GROUP_LEN, "hardkey-%i", 
		   *hardkey_data->hardkey_index);
	irreco_gkeyfile_set_string(hardkey_data->keyfile, group, "layout", 
		irreco_button_layout_get_name(hardkey_data->layout));
	g_key_file_set_integer(hardkey_data->keyfile, group, 
			       "chain-id", chain_id);
	irreco_gkeyfile_set_guint(hardkey_data->keyfile, group, 
				  "key-value", keyval);
	
	*hardkey_data->hardkey_index = *hardkey_data->hardkey_index + 1;
	IRRECO_RETURN
}

void irreco_config_save_button_to_keyfile(GKeyFile * keyfile, 
					  IrrecoButton * button,
					  gint * button_index,
					  gint * command_index)
{
	gchar group[IRRECO_CONFIG_GROUP_LEN];
	IRRECO_ENTER
	
	g_snprintf(group, IRRECO_CONFIG_GROUP_LEN, "button-%i", *button_index);
	irreco_gkeyfile_set_string(keyfile, group, "layout", 
		irreco_button_layout_get_name(button->irreco_layout));
	irreco_gkeyfile_set_string(keyfile, group, "title", button->title);
	g_key_file_set_integer(keyfile, group, "index", button->index);
	g_key_file_set_integer(keyfile, group, "command-chain-id", 
			       button->cmd_chain_id);
	g_key_file_set_integer(keyfile, group, "x", button->x);
	g_key_file_set_integer(keyfile, group, "y", button->y);
	
	/* It is valid for a button to have NULL style. */
	if (button->style != NULL) {
		irreco_gkeyfile_set_string(keyfile, group, "style", 
				      button->style->name);
	}

	IRRECO_RETURN
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Save / Read active layout.                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

gboolean irreco_config_save_active_layout(IrrecoButtonLayout * irreco_layout)
{
	GKeyFile *keyfile;
	gboolean success;
	IRRECO_ENTER
	
	keyfile = g_key_file_new();
	if (irreco_layout != NULL) {
		irreco_gkeyfile_set_string(keyfile, "active-layout", "name", 
			irreco_button_layout_get_name(irreco_layout));
	}
	success = irreco_gkeyfile_write_to_config_file(keyfile, "irreco", 
					      irreco_config_active_layout_file);
	g_key_file_free(keyfile);
	IRRECO_RETURN_BOOL(success);
}

IrrecoButtonLayout *irreco_config_read_active_layout(IrrecoData * irreco_data)
{
	IrrecoKeyFile *keyfile = NULL;
	gchar *config_dir = NULL;
	gchar *config_file = NULL;
	gchar *name = NULL;
	IrrecoButtonLayout *irreco_layout = NULL;
	IRRECO_ENTER
	
	/* Attempt to read layout name, and find layout with the same name. */
	config_dir = irreco_get_config_dir("irreco");
	config_file = irreco_get_config_file("irreco", 
					     irreco_config_active_layout_file);
	keyfile = irreco_keyfile_create(config_dir, config_file, NULL);
	if (config_dir != NULL && config_file != NULL && keyfile != NULL
	    && irreco_keyfile_set_group(keyfile, "active-layout")
	    && irreco_keyfile_get_str(keyfile, "name", &name)) {
		    
		IRRECO_DEBUG("Read active layout name \"%s\" from \"%s\".\n",
			  name, irreco_config_active_layout_file);
			  
		if (irreco_string_table_get(irreco_data->irreco_layout_array, 
					    name, (gpointer *) &irreco_layout)){
			IRRECO_DEBUG("Found matching layout.\n");
		} else {
			IRRECO_DEBUG("No matching layout found.\n");
		}
	}
	
	irreco_keyfile_destroy(keyfile);
	g_free(name);
	g_free(config_dir);
	g_free(config_file);
	
	/* If we could not find correct active layout, 
	   attempt to get some layout.*/
	if (irreco_layout == NULL) {
		irreco_string_table_index(irreco_data->irreco_layout_array, 0, 
					  NULL, (gpointer *) &irreco_layout);
	}
	IRRECO_RETURN_PTR(irreco_layout);
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Save / Read backend configuration.                                         */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

gboolean irreco_config_save_backends(IrrecoBackendManager * backend_manager)
{
	gint i = 0;
	GKeyFile *keyfile;
	gboolean success;
	gchar group[IRRECO_CONFIG_GROUP_LEN];
	IRRECO_ENTER
	
	keyfile = g_key_file_new();
	IRRECO_BACKEND_MANAGER_FOREACH_INSTANCE(backend_manager, instance)
		g_snprintf(group, IRRECO_CONFIG_GROUP_LEN, "instance-%i", ++i);
		irreco_gkeyfile_set_string(keyfile, group, "library", 
					 instance->lib->filename);
		irreco_gkeyfile_set_string(keyfile, group, "config", 
					 instance->config);
		irreco_gkeyfile_set_string(keyfile, group, "name", 
					 instance->name);
	IRRECO_BACKEND_MANAGER_FOREACH_END
	
	success = irreco_gkeyfile_write_to_config_file(keyfile, "irreco", 
						    irreco_config_backend_file);
	g_key_file_free(keyfile);
	IRRECO_RETURN_BOOL(success);
}

gboolean irreco_config_read_backends(IrrecoBackendManager * manager)
{
	guint i;
	IrrecoKeyFile *keyfile = NULL;
	gchar *config_dir = NULL;
	gchar *config_file = NULL;
	gchar** groups = NULL;
	gsize group_count = 0;
	IRRECO_ENTER
	
	config_dir = irreco_get_config_dir("irreco");
	config_file = irreco_get_config_file("irreco", 
					     irreco_config_backend_file);
	keyfile = irreco_keyfile_create(config_dir, config_file, NULL);
	if (config_dir == NULL || config_file == NULL || keyfile == NULL) {
		g_free(config_dir);
		g_free(config_file);
		irreco_keyfile_destroy(keyfile);
		IRRECO_RETURN_BOOL(FALSE);
	}
	
	IRRECO_PRINTF("Reading backend configuration file \"%s\"\n", 
		      config_file);
	groups = g_key_file_get_groups(keyfile->keyfile, &group_count);
	
	/* Read instances. */
	for (i = 0; i < group_count; i++) {
		gchar *group = groups[i];
		if (g_str_has_prefix(group, "instance")) {
			IRRECO_PRINTF("Reading instance group \"%s\"\n", group);
			irreco_keyfile_set_group(keyfile, group);
			irreco_config_read_backend_from_keyfile(manager, 
								keyfile);
		}
	}
	
	g_strfreev(groups);
	irreco_keyfile_destroy(keyfile);
	g_free(config_dir);
	g_free(config_file);
	IRRECO_RETURN_BOOL(TRUE);
}

void irreco_config_read_backend_from_keyfile(IrrecoBackendManager * manager,
					     IrrecoKeyFile * keyfile)
{
	gchar *library = NULL;
	gchar *config = NULL;
	gchar *name = NULL;
	IrrecoBackendLib *lib;
	IRRECO_ENTER
	
	/* Read variables. */
	if (!irreco_keyfile_get_str(keyfile, "library", &library) ||
	    !irreco_keyfile_get_str(keyfile, "config", &config) ||
	    !irreco_keyfile_get_str(keyfile, "name", &name)) {
		g_free(library);
		g_free(config);
		g_free(name);
		IRRECO_RETURN
	}
	
	/* Find library. */
	if (!irreco_backend_manager_find_lib(manager, library, &lib)) {
		IRRECO_ERROR("Cannot find backend library \"%s\"\n", library);
		g_free(library);
		g_free(config);
		g_free(name);
		IRRECO_RETURN
	}
	g_free(library);
	
	/* Create instance. */
	irreco_backend_manager_create_instance(manager, lib, name, config);
	g_free(name);
	g_free(config);

	IRRECO_RETURN
}




