/*
 * irreco - Ir Remote Control
 * Copyright (C) 2008 Joni Kokko (t5kojo01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_theme_bg.h"

/**
 * @addtogroup IrrecoThemeBg
 * @ingroup Irreco
 *
 * Contains information of background.
 *
 * @{ 
 */

/**
 * @file
 * Source file of @ref IrrecoThemeBg.
 */
 
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Construction & Destruction
 * @{
 */

IrrecoThemeBg *irreco_theme_bg_new()
{
	IrrecoThemeBg *self;
	IRRECO_ENTER
	
	self = g_slice_new0(IrrecoThemeBg);

	self->image_name = g_string_new("");
	self->image_path = g_string_new("");

	IRRECO_RETURN_PTR(self);
}

void irreco_theme_bg_free(IrrecoThemeBg *self)
{
	IRRECO_ENTER
			
	g_assert(self != NULL);
	
	g_string_free(self->image_name, TRUE);
	self->image_name = NULL;

	g_string_free(self->image_path, TRUE);
	self->image_path = NULL;
	
	g_slice_free(IrrecoThemeBg, self);
	self = NULL;

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Public Functions                                                           */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Public Functions
 * @{
 */

void irreco_theme_bg_set(IrrecoThemeBg *self,
			 const char *image_name,
			 const char *image_path)
{
	IRRECO_ENTER
	
	g_string_printf(self->image_name, "%s", image_name);
	g_string_printf(self->image_path, "%s", image_path);
	
	IRRECO_RETURN
}

void irreco_theme_bg_print(IrrecoThemeBg *self)
{
	IRRECO_ENTER

	IRRECO_DEBUG("--------------------------------------------\n");
	IRRECO_DEBUG("Background-image: %s\n", self->image_name->str);
	IRRECO_DEBUG("Background-image_path: %s\n", self->image_path->str);
	IRRECO_DEBUG("--------------------------------------------\n");

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/** @} */
