/*
 * irreco - Ir Remote Control
 * Copyright (C) 2008 Joni Kokko (t5kojo01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_theme_button.h"

/**
 * @addtogroup IrrecoThemeButton
 * @ingroup Irreco
 *
 * Contains information of button.
 *
 * @{ 
 */

/**
 * @file
 * Source file of @ref IrrecoThemeButton.
 */
 
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Construction & Destruction
 * @{
 */

IrrecoThemeButton *irreco_theme_button_new()
{
	IrrecoThemeButton *self;
	IRRECO_ENTER
	
	self = g_slice_new0(IrrecoThemeButton);

	self->style_name = g_string_new("");
	self->name = g_string_new("");
	self->image_up = g_string_new("");
	self->image_down = g_string_new("");
	self->text_format_up = g_string_new("");
	self->text_format_down = g_string_new("");
	
	IRRECO_RETURN_PTR(self);
}

void irreco_theme_button_free(IrrecoThemeButton *self)
{
	IRRECO_ENTER
			
	g_assert(self != NULL);
	
	g_string_free(self->style_name, TRUE);
	self->style_name = NULL;

	g_string_free(self->name, TRUE);
	self->name = NULL;
	
	g_string_free(self->image_up, TRUE);
	self->image_up = NULL;
	
	g_string_free(self->image_down, TRUE);
	self->image_down = NULL;
	
	g_string_free(self->text_format_up, TRUE);
	self->text_format_up = NULL;
	
	g_string_free(self->text_format_down, TRUE);
	self->text_format_down = NULL;
	
	g_slice_free(IrrecoThemeButton, self);
	self = NULL;

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Public Functions                                                           */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Public Functions
 * @{
 */

void irreco_theme_button_set(IrrecoThemeButton *self,
			     const char *style_name,
			     const char *name,
			     gboolean	 allow_text,
			     const char *image_up,
			     const char *image_down,
			     const char *text_format_up,
			     const char *text_format_down,
			     gint	 text_padding)
{
	IRRECO_ENTER
	
	if (style_name != NULL) {
		g_string_printf(self->style_name, "%s", style_name);
	}
	if (name != NULL) {
		g_string_printf(self->name, "%s", name);
	}
	self->allow_text = allow_text;
	if (image_up != NULL) {
		g_string_printf(self->image_up, "%s", image_up);
	}
	if (image_down != NULL) {
		g_string_printf(self->image_down, "%s", image_down);
	} else {
		g_string_printf(self->image_down, "%s", image_up);
	}
	if (text_format_up != NULL) {
		g_string_printf(self->text_format_up, "%s", text_format_up);
	}
	if (text_format_down != NULL) {
		g_string_printf(self->text_format_down, "%s", text_format_down);
	}
	self->text_padding = text_padding;
	
	IRRECO_RETURN
}

void irreco_theme_button_print(IrrecoThemeButton *self)
{
	IRRECO_ENTER

	IRRECO_DEBUG("--------------------------------------------\n");
	IRRECO_DEBUG("style_name: %s\n", self->style_name->str);
	IRRECO_DEBUG("button_name: %s\n", self->name->str);
	IRRECO_DEBUG("allow_text: %d\n", self->allow_text);
	IRRECO_DEBUG("image_up: %s\n",self->image_up->str);
	IRRECO_DEBUG("image_down: %s\n", self->image_down->str);
	IRRECO_DEBUG("text_format_up: %s\n", self->text_format_up->str);
	IRRECO_DEBUG("text_format_down: %s\n", self->text_format_down->str);
  	IRRECO_DEBUG("text_padding: %d\n", self->text_padding);
	IRRECO_DEBUG("--------------------------------------------\n");

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/** @} */
